@@ -92,8 +92,8 @@ void AddCovs(int obs, int nobs, int npars, int *ncovs,
 		newpars[i] = link(newpars[i]);
 		for (j = 0; j < ncovs[i]; ++j) {
 			x =  cov[MI(obs, whichcov[j]-1, nobs)];
-		    newpars[i] += coveffect[k] * x;
-		    ++k;
+			newpars[i] += coveffect[k] * x;
+			++k;
 		}
 		newpars[i] = invlink(newpars[i]);
 		*totcovs += ncovs[i];
@@ -135,9 +135,9 @@ void GetCensored (double obs, cmodel *cm, int *nc, double **states)
 
 /*
   Calculate p (obs curr | true i) for hidden Markov models 
-  If no censoring (nc==1), and observation is not necessarily of the true state (!obstrue), 
-  then this is just the HMM outcome model. 
-  For censored outcomes, there is no misclassification / HMM outcome model. 
+  If observation is not necessarily of the true state (!obstrue), 
+  then this is just the HMM outcome probability (summed over censor set if necessary) 
+  If obstrue, this observation is not misclassified. 
   e.g. censor set 1,2,3,    state set 1,2,3,4, 
   pout =   if i in curr 1, else 0 
 */
@@ -147,8 +147,10 @@ void GetOutcomeProb(double *pout, double *curr, int nc, double *newpars, hmodel
     int i, j;
     for (i=0; i<qm->nst; ++i) {
 	pout[i] = 0;
-	if (nc == 1 && !obstrue)
-	    pout[i] = (HMODELS[hm->models[i]])(curr[0], &(newpars[hm->firstpar[i]]));
+	if (!obstrue) {
+	    for (j=0; j<nc; ++j)
+		pout[i] += (HMODELS[hm->models[i]])(curr[j], &(newpars[hm->firstpar[i]]));
+	}
 	else { 
 	    for (j=0; j<nc; ++j) 
 		if ((int) curr[j] == i+1) 
@@ -319,8 +321,9 @@ void update_likcensor(int obsno, double *prev, double *curr, int np, int nc,
 		    newp[i] += cump[j] * contrib;
 		    
 		}
-		else 
+		else {
 		    newp[i] += cump[j] * pmat[MI((int) prev[j]-1, (int) curr[i]-1, qm->nst)];
+		}
 	    }
 	}
     normalize(newp, cump, nc, lweight); 
@@ -375,13 +378,13 @@ double liksimple(msmdata *d, qmodel *qm, qcmodel *qcm,
     for (i=0; i < d->nobs; ++i)
 	{
 	    R_CheckUserInterrupt();
-	    if ((i==0) || (d->whicha[i] != d->whicha[i-1])) {
-		/* we have a new timelag/covariates combination. Recalculate the 
+	    if ((i==0) || (d->whicha[i] != d->whicha[i-1]) || (d->obstype[i] != d->obstype[i-1])) {
+		/* we have a new timelag/covariates/obstype combination. Recalculate the 
 		   P matrix for this */
 		AddCovs(i, d->nobs, qm->npars, qcm->ncovs, qm->intens, newintens,
 			qcm->coveffect, d->cov, d->whichcov, &totcovs,
 			log, exp);
-		Pmat(pmat, d->timelag[i], newintens, qm->npars, qm->ivector, qm->nst, (d->obstype[i] == OBS_EXACT), qm->analyticp, qm->iso, qm->perm,  qm->qperm, 0);
+		Pmat(pmat, d->timelag[i], newintens, qm->npars, qm->ivector, qm->nst, (d->obstype[i] == OBS_EXACT), qm->analyticp, qm->iso, qm->perm,  qm->qperm, i==37);
 	    }
 	    if (d->obstype[i] == OBS_DEATH)
 		contrib = pijdeath(d->fromstate[i], d->tostate[i], pmat, newintens, qm->ivector, qm->nst);
@@ -389,7 +392,7 @@ double liksimple(msmdata *d, qmodel *qm, qcmodel *qcm,
 		contrib = pmat[MI(d->fromstate[i], d->tostate[i], qm->nst)];
 	    lik += d->nocc[i] * log(contrib);
 #ifdef DEBUG
-	    printf("obs %d, from %d, to %d, time %lf, ", i, d->fromstate[i], d->tostate[i], d->timelag[i]);
+	    printf("obs %d, from %d, to %d, time %lf, obstype %d, ", i, d->fromstate[i], d->tostate[i], d->timelag[i], d->obstype[i]);
 	    for (j = 0; j < qcm->ncovs[0]; ++j)
 		printf("cov%d %lf, ", j, d->cov[MI(i, d->whichcov[j]-1, d->nobs)]);
 	    printf("nocc %d, con %lf, lik %lf\n", d->nocc[i], log(contrib), lik);
@@ -616,8 +619,8 @@ void derivsimple(msmdata *d, qmodel *qm, qcmodel *qcm,
     for (i=0; i < d->nobs; ++i)
 	{
 	    R_CheckUserInterrupt();
-	    if ((i==0) || (d->whicha[i] != d->whicha[i-1])) {
-		/* we have a new timelag/covariates combination. Recalculate the 
+	    if ((i==0) || (d->whicha[i] != d->whicha[i-1]) || (d->obstype[i] != d->obstype[i-1])) {
+		/* we have a new timelag/covariates/obstype combination. Recalculate the 
 		   P matrix and its derivatives for this */
 		GetCovData(i, d->cov, d->whichcov, x, qcm->ncovs[0], d->nobs);
 		AddCovs(i, d->nobs, np, qcm->ncovs, qm->intens, newintens,
@@ -670,9 +673,10 @@ void derivsimple_subj(msmdata *d, qmodel *qm, qcmodel *qcm,
 		    for (p = 0; p < ndp+ndc; ++p) {
 			deriv[MI(pt,p,d->npts)] = 0;
 		    }
-		    GetCovData(i, d->covobs, d->whichcov, x, qcm->ncovs[0], d->nobs);
-		    AddCovs(i, d->nobs, np, qcm->ncovs, qm->intens, newintens,
+		    GetCovData(i, d->covobs, d->whichcov, x, qcm->ncovs[0], d->n);
+		    AddCovs(i, d->n, np, qcm->ncovs, qm->intens, newintens,
 			    qcm->coveffect, d->covobs, d->whichcov, &totcovs, log, exp);
+		    
 		    dt = d->time[i] - d->time[i-1];
 		    from = fprec(d->obs[i-1] - 1, 0); /* convert state outcome to integer */
 		    to = fprec(d->obs[i] - 1, 0);
@@ -689,12 +693,9 @@ void derivsimple_subj(msmdata *d, qmodel *qm, qcmodel *qcm,
 			for (p = 0; p < ndp+ndc; ++p)
 			    dcontrib[p] = dpmat[MI3(from, to, p, qm->nst, qm->nst)];		
 		    }
-/*  		    printf("pt=%d,obs=%d,from=%d,to=%d,dt=%4.3lf,contrib=%lf,",pt,i,from,to,dt,contrib); */
 		    for (p = 0; p < ndp+ndc; ++p) {
-/*  			printf("d%d = %4.3f,",p,dcontrib[p]); */
-			deriv[MI(pt,p,d->npts)] += dcontrib[p] / contrib; /* on loglik scale not -2*loglik */
+		      deriv[MI(pt,p,d->npts)] += dcontrib[p] / contrib; /* on loglik scale not -2*loglik */
 		    }
-/*  		    printf("\n"); */
 		}
 		for (p = 0; p < ndp+ndc; ++p)
 		    deriv[MI(pt,p,d->npts)] *= -2;
@@ -775,7 +776,8 @@ void msmCEntry(
 	       int *nintens,      /* number of intensity parameters */
 	       int *ndintens,      /* number of distinct intensity parameters */
 	       int *ndcovpars,      /* number of distinct covariate parameters */
-	       int *nobs,         /* number of observations in data set */
+	       int *nobs,         /* number of observations in data set (hmm/cens) or number of aggregated transitions (standard) */
+	       int *n,         /* number of observations in data set (used for derivs by individual in standard models. no of rows of covobsvec) */
 	       int *npts,         /* number of individuals in data set */
 	       int *ncovs,        /* number of covariates on transition rates */
 
@@ -797,7 +799,7 @@ void msmCEntry(
   d.cov = covvec;  d.covobs = covobsvec;  d.nocc = nocc;  d.whicha = whicha; d.obstype = obstype;  d.obstrue = obstrue; 
   d.whichcov = whichcov;  d.whichcovh = whichcovh;  d.whichcovi=whichcovi;
   d.subject = subjvec; d.time = timevec; d.obs = obsvec; d.firstobs = firstobs;
-  d.nobs = *nobs;  d.npts = *npts;
+  d.nobs = *nobs;  d.n = *n; d.npts = *npts;
 
   qm.nst = *nst; qm.npars = *nintens; qm.ndpars = *ndintens; qm.ivector = qvector; qm.intens = intens; 
   qm.analyticp = *analyticp; qm.iso = *iso; qm.perm = perm; qm.qperm = qperm; qm.constr = qconstraint;    
